@@ -89,7 +89,7 @@ def multiload(locations):
     return sims, lins
 
 
-def multiplot(sims, suffixes='', dashes=[(1, 0), (3, 3), (1, 1), (3, 2, 1, 2)],
+def multiplot(sims, suffixes='', dashes=[(None, None), (3, 3), (1, 1), (3, 2, 1, 2)],
               **kwargs):
     """Plot data from multiple simulations in 2D Cartesian coordinates.
 
@@ -109,7 +109,7 @@ def multiplot(sims, suffixes='', dashes=[(1, 0), (3, 3), (1, 1), (3, 2, 1, 2)],
          If *suffixes* is None, then no suffix will be used.  If it is *None*,
          then the base filenames will be used.
 
-    - *\*\*kwargs*: Propagated to  :meth:`simres.SimRes.plot` (and thus to
+    - *\*\*kwargs*: Additional arguments for   :meth:`simres.SimRes.plot` (and thus to
       :meth:`base.plot` and finally :meth:`matplotlib.pyplot.plot`)
 
          The *dashes* sequence is iterated across all plots.
@@ -130,7 +130,7 @@ def multiplot(sims, suffixes='', dashes=[(1, 0), (3, 3), (1, 1), (3, 2, 1, 2)],
        ...           suffixes=['L.L = %.0f H' % sim.get_IV('L.L')
        ...                     for sim in sims], # Read legend parameters.
        ...           ynames1='L.i', ylabel1="Current") # doctest: +ELLIPSIS
-       (<matplotlib.axes.AxesSubplot object at 0x...>, None)
+       (<matplotlib.axes._subplots.AxesSubplot object at 0x...>, None)
 
        >>> saveall()
        Saved examples/ChuaCircuits.pdf
